% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HmscRandomLevel.R
\name{HmscRandomLevel}
\alias{HmscRandomLevel}
\title{Create an \code{Hmsc} random level}
\usage{
HmscRandomLevel(
  sData = NULL,
  sMethod = "Full",
  distMat = NULL,
  xData = NULL,
  units = NULL,
  N = NULL,
  nNeighbours = 10,
  sKnot = NULL,
  longlat = FALSE
)
}
\arguments{
\item{sData}{a matrix or a dataframe containing spatial or temporal
coordinates of units of the random level, or a similar
\code{SpatialPoints} structure of the \CRANpkg{sp} package. If
spatial coordinates are unprojected longitude and latitude,
great circle distances will be calculated internally. All
spatial locations should be unique. If you have several
observations in the same point, they should be identified by
the random levels.}

\item{sMethod}{a string specifying which spatial method to be
used. Possible values are \code{"Full"}, \code{"GPP"} and
\code{"NNGP"}}

\item{distMat}{a distance matrix containing the distances between
units of the random level, with unit names as rownames, or a
\code{\link{dist}} structure with location
Labels. \code{distMat} cannot be used with \code{"GPP"} spatial
model.}

\item{xData}{a dataframe containing the covariates measured at the
units of the random level for covariate-dependent associations}

\item{units}{a vector, specifying the names of the units of a
non-structured level}

\item{N}{number of unique units on this level}

\item{nNeighbours}{a scalar specifying the number of neighbours to
be used in case the spatial method is set to \code{NNGP}. Only
positive values smaller than the total number of plots are
allowed.}

\item{sKnot}{a dataframe containing the knot locations to be used
for the Gaussian predictive process if \code{sMethod} is set to
\code{"GPP"}. Suitable data can be produced with
\code{\link{constructKnots}}. The knot locations shall not
duplicate \code{sData}.}

\item{longlat}{Interpret coordinate data \code{sData} as longitude
and latitude in decimal degrees. If this is \code{TRUE}, great
circle distances will be used instead of Euclidean distances.}
}
\value{
a \code{HmscRandomLevel}-class object that can be used for \code{Hmsc}-class object construction
}
\description{
Specifies the structure of a random factor, including
    whether the random factor is assumed to be spatially explicit
    or not, the spatial coordinates and the potential structure of
    covariate-dependent random factors.
}
\details{
Only one of \code{sData}, \code{distMat}, \code{xData}, \code{units} and \code{N} arguments can be
  provided.

  As a good practice, we recommend to specify all available units for a random level, even if some of those are not
  used for training the model.
}
\examples{
# Setting a random level with 50 units
rL = HmscRandomLevel(units=TD$studyDesign$sample)

# Setting a spatial random level
rL = HmscRandomLevel(sData=TD$xycoords)

# Setting a covariate-dependent random level.
rL = HmscRandomLevel(xData=data.frame(x1=rep(1,length(TD$X$x1)),x2=TD$X$x2))

}
\seealso{
\code{\link{setPriors.Hmsc}} to change the default priors
    of an existing \code{HmscRandomLevel} object.
}
